/**************************************************************************/
/* Copyright 1995 National Instruments Corporation.  All Rights Reserved. */
/**************************************************************************/

#include "\lw\include\lwsystem.h"
#include "\lw\include\gpib.h"
#include "\lw\include\formatio.h"
#include "kptma488.h"

/*= UTILITY ROUTINES ======================================================*/
short tma488_invalid_integer_range (short, short, short, short);
short tma488_invalid_real_range (double, double, double, short);
short tma488_device_closed (void);
short tma488_read_data (char *, short);
short tma488_write_data (char *, short);
void tma488_setup_arrays (void);

/*= Kepco TMA488-27========================================================*/
/* LabWindows 2.0 Instrument Driver                                        */
/* Original Release: Jan 1992                                              */
/* By: Peter Blanco, Kepco Inc.                                            */
/*     Ph (718) 461-7000    Fax (718) 767-1102                             */
/* Originally written in C                                                 */
/* Modification History:                                                   */
/*                                                                         */
/*  Sept 1995:      Revised to work with TMA 4882-27 by adding "CIIL"      */
/*                  instruction in init function. Also fixed problem in    */
/*                  read function which prevented proper reading of output */
/*                  values.                                                */
/*                                                                         */
/*                  Modified by Peter Blanco, Kepco Inc.                   */
/*                                                                         */
/*  Oct 1995:       Changed the GPIB Library calls so that they check for  */
/*                  the errorbit being set to indicate an error (& 0x8000) */
/*                  instead of checking for a negative return value.       */
/*                  Also because the instrument does not support an ID     */
/*                  Query added ibln to insure that there is an            */
/*                  instrument at the specified address on the Bus, and if */
/*                  there is not, set bd to -1 so that _device_closed will */
/*                  return.                                                */
/*                                                                         */
/*                  Modified by JRO, National Instruments, Austin, TX.     */
/*                  Ph (800) 433-3488    Fax (512) 794-5678                */
/*                                                                         */
/*=========================================================================*/

/*= STATIC VARIABLES ======================================================*/
/* bd array contains the device descriptor returned by OpenDev.            */
/* cmd is a buffer for GPIB I/O strings.                                   */
/* tma488_err is the error variable for the instrument module              */
/*=========================================================================*/
static short bd;
static char cmd[50];
static short tma488_err;

/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS ===================================*/
static char * func[6];

/*=========================================================================*/
/* Function: Initialize                                                    */
/* Purpose:  This function opens the instrument                            */
/*=========================================================================*/
short tma488_init (short addr)
{
    short resp;

    if (tma488_invalid_integer_range (addr, 0, 30, -1) != 0)
        return tma488_err;

    /*     If device has not been opened (bd = 0), open it.    */
    if (bd <= 0)  {
        CloseInstrDevs ("tma488");
        bd = OpenDev ("", "tma488");
        if (bd <= 0)  {
            tma488_err = 220;
            return tma488_err;
        }
    }
    /*     Change the primary address of the device       */
    if (ibpad (bd, addr) & 0x8000)  {
        tma488_err = 233;
        return tma488_err;
    }

    /* The ID Query "VER" response varies with model,so ibln is used to */
    /* insure that there is an instrument at the specified address on   */
    /* the Bus. Set bd to -1 so that _device_closed will return if no   */
    /* device is  found at the specified address.                       */

    if (ibln (bd, addr, 0, &resp) & 0x8000) {
        bd = -1;
        tma488_err = 223;
        return tma488_err;
    }
    if (resp == 0) {
        bd = -1;
        tma488_err = 223;
        return tma488_err;
    }

    if (tma488_write_data ("CIIL\n", 5) != 0)
        return tma488_err;
    if (tma488_write_data ("GAL\n", 4) != 0)
        return tma488_err;
    if (tma488_write_data ("F0\n", 3) != 0)
        return tma488_err;
    if (tma488_write_data ("H1\n", 3) != 0)
        return tma488_err;

       /*************************************************************
       GAL is Go Alternate Language, which enables non-CIIL commands.
       F0 allows the FTH command to return the output reading
       even if an error condition exists.
       H1 allows the GPIB bus to be held up to 2 secs. if the
       output of a supply is not ready to be read.
       This is more efficient than using the INX return value
       to determine when the output is ready. The default
       states for these commands are F1 and H0. The default
       states were necessary in order to comply with the
       original MATE/CIIL command protocol.
       *************************************************************/

    tma488_setup_arrays ();
    return tma488_err;
}

/*=========================================================================*/
/* Function: Set Output                                                    */
/* Purpose:  This function sets the mode and the voltage and current       */
/*           levels for the selected channel.                              */
/*=========================================================================*/
short tma488_set (short chn, short mod, double volt, double curr)
{
    if (tma488_device_closed () != 0)
        return tma488_err;
    if (tma488_invalid_integer_range (chn, 0, 30, -1) != 0)
        return tma488_err;
    if (tma488_invalid_integer_range (mod, 0, 1, -2) != 0)
        return tma488_err;

    Fmt (cmd, "%s<FNC DCS :CH%i[b2] SET %s %f SET %s %f\r\n", chn, func[mod * 2], volt, func[mod * 2 + 1], curr);
    if (tma488_write_data (cmd, NumFmtdBytes ()) != 0)
        return tma488_err;

    return tma488_err;
}

/*=========================================================================*/
/* Function: Output On/Off                                                 */
/* Purpose:  This function sets the output of the selected channel to on   */
/*           or off.                                                       */
/*=========================================================================*/
short tma488_output (short chn, short mod)
{
    if (tma488_device_closed () != 0)
        return tma488_err;
    if (tma488_invalid_integer_range (chn, 0, 30, -1) != 0)
        return tma488_err;
    if (tma488_invalid_integer_range (mod, 0, 1, -2) != 0)
        return tma488_err;

    Fmt (cmd, "%s<%s :CH%i[b2]\r\n", func[mod + 4], chn);
    if (tma488_write_data (cmd, NumFmtdBytes ()) != 0)
        return tma488_err;

    return tma488_err;
}

/*=========================================================================*/
/* Function: Read                                                          */
/* Purpose:  This function reads the current output voltage or current of  */
/*           the selected channel.                                         */
/*=========================================================================*/
short tma488_read (short chn, short mod, double *val)
{
    if (tma488_device_closed () != 0)
        return tma488_err;
    if (tma488_invalid_integer_range (chn, 0, 30, -1) != 0)
        return tma488_err;
    if (tma488_invalid_integer_range (mod, 0, 1, -2) != 0)
        return tma488_err;

    Fmt (cmd, "%s<FNC DCS %s :CH%i[b2]\r\n", func[mod * 3], chn);
    if (tma488_write_data (cmd, NumFmtdBytes ()) != 0)
        return tma488_err;
    Fmt (cmd, "%s<INX %s\r\n", func[mod * 3]);
    if (tma488_write_data (cmd, NumFmtdBytes ()) != 0)
        return tma488_err;
    if (tma488_read_data (cmd, 40) != 0)
        return tma488_err;
    if (Scan (cmd, "%f", val) != 1)  {
        tma488_err = 236;
        return tma488_err;
    }

    /* The INX response may be discarded.It is only used, to comply with
       the CIIl language standard */

    Fmt (cmd, "%s<FTH %s\r\n", func[mod * 3]);
    if (tma488_write_data (cmd, NumFmtdBytes ()) != 0)
        return tma488_err;
    if (tma488_read_data (cmd, 40) != 0)
        return tma488_err;
    if (Scan (cmd, "%f", val) != 1)  {
        tma488_err = 236;
        return tma488_err;
    }
    return tma488_err;
}

/*=========================================================================*/
/* Function: Reset                                                         */
/* Purpose:  This function resets the instrument to its default state.     */
/*=========================================================================*/
short tma488_reset (short chn)
{
    if (tma488_device_closed () != 0)
        return tma488_err;
    if (tma488_invalid_integer_range (chn, 0, 30, -1) != 0)
        return tma488_err;

    Fmt (cmd, "%s<RST DCS :CH%i[b2]\r\n", chn);
    if (tma488_write_data (cmd, NumFmtdBytes ()) != 0)
        return tma488_err;

    return tma488_err;
}

/*=========================================================================*/
/* Function: Status                                                        */
/* Purpose:  This function reads the status string from the instrument.    */
/*=========================================================================*/
short tma488_status (char *stat)
{
    if (tma488_device_closed () != 0)
        return tma488_err;

    Fmt (cmd, "%s<STA\r\n");
    if (tma488_write_data (cmd, NumFmtdBytes ()) != 0)
        return tma488_err;
    if (tma488_read_data (stat, 40) != 0)
        return tma488_err;

    return tma488_err;
}

/*=========================================================================*/
/* Function: Command                                                       */
/* Purpose:  This function writes a string to the intstrument.             */
/*=========================================================================*/
short tma488_cmnd (char *cmnd)
{
    if (tma488_device_closed () != 0)
        return tma488_err;

    Fmt (cmd, "%s<%s\r\n", cmnd);
    if (tma488_write_data (cmd, NumFmtdBytes ()) != 0)
        return tma488_err;

    return tma488_err;
}

/*=========================================================================*/
/* Function: Recieve String                                                */
/* Purpose:  This function reads a string from the instrument.             */
/*=========================================================================*/
short tma488_rec_str (char *strng)
{
    if (tma488_device_closed () != 0)
        return tma488_err;

    Fmt (strng, "%s<                                       ");
    if (tma488_read_data (strng, 40) != 0)
        return tma488_err;

    return tma488_err;
}

/*=========================================================================*/
/* Function: Close                                                         */
/* Purpose:  This function removes the instrument from the GPIB device     */
/*           table and sets bd = 0.                                        */
/*=========================================================================*/
short tma488_close (void)
{
    if (tma488_device_closed () != 0)
        return tma488_err;

    /*     Close the device       */
    if (CloseDev (bd) < 0)
        tma488_err = 221;
    else
        tma488_err = 0;

    bd = 0;
    return tma488_err;
}

/*=========================================================================*/
/* Function: Invalid Integer Range                                         */
/* Purpose:  This function checks an integer to see if it lies between a   */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
short tma488_invalid_integer_range (short val, short min, short max, short err_code)
{
    if (val < min || val > max)
        tma488_err = err_code;
    else
        tma488_err = 0;

    return tma488_err;
}

/*=========================================================================*/
/* Function: Invalid Real Range                                            */
/* Purpose:  This function checks a real number to see if it lies between  */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
short tma488_invalid_real_range (double val, double min, double max, short err_code)
{
    if (val < min || val > max)
        tma488_err = err_code;
    else
        tma488_err = 0;

    return tma488_err;
}

/*=========================================================================*/
/* Function: Device Closed                                                 */
/* Purpose:  This function checks to see if the module has been            */
/*           initialized.  If the device has not been opened, set the      */
/*           global error variable to 232, 0 otherwise. The return value   */
/*           is equal to the global error value.                           */
/*=========================================================================*/
short tma488_device_closed (void)
{
    if (bd <= 0)
        tma488_err = 232;
    else
        tma488_err = 0;

    return tma488_err;
}

/*=========================================================================*/
/* Function: Read Data                                                     */
/* Purpose:  This function reads a buffer of data from the instrument. The */
/*           return value is equal to the global error variable.           */
/*=========================================================================*/
short tma488_read_data (char *buf, short cnt)
{
    if (ibrd (bd, buf, (long)cnt) & 0x8000)
        tma488_err = 231;
    else
        tma488_err = 0;

    return tma488_err;
}

/*=========================================================================*/
/* Function: Write Data                                                    */
/* Purpose:  This function writes a buffer of data to the instrument. The  */
/*           return value is equal to the global error variable.           */
/*=========================================================================*/
short tma488_write_data (char *buf, short cnt)
{
    if (ibwrt (bd, buf, (long)cnt) & 0x8000)
        tma488_err = 230;
    else
        tma488_err = 0;

    return tma488_err;
}

/*=========================================================================*/
/* Function: Setup Arrays                                                  */
/* Purpose:  This function is called by the init routine to initialize     */
/*           global arrays.                                                */
/*=========================================================================*/
void tma488_setup_arrays (void)
{

    func[0] = "VOLT";
    func[1] = "CURL";
    func[2] = "VLTL";
    func[3] = "CURR";
    func[4] = "OPN";
    func[5] = "CLS";
}

/*=== THE END =============================================================*/
